module MinimizesPartition
  class MinimizesPartition
        attr_accessor :original_list, :word_list , :already_tried_letters

        def word_list=(list)
          @original_list = list.dup
        end

        def new_game(guesses_left)
          @already_tried_letters = []
          @last_guess_correct = true
          @word_list = @original_list
        end

        def guess(word, guesses_left)
          filter_word_list_by_word_pattern(word)  if @last_guess_correct        
          letter_with_least_worst_case 
        end

        def incorrect_guess(guess)
          @last_guess_correct = false
          @already_tried_letters << guess
          filter_word_list_by_incorrect_letter guess
        end

        def correct_guess(guess)
          @last_guess_correct = true
          @already_tried_letters << guess
        end

        def fail(reason)
        end

        def game_result(result, word)
        end

        def letter_with_least_worst_case
          least_worst_case_so_far = @word_list.size + 1
          potential_remaining_letters.each do |letter|
            matches,non_matches = @word_list.partition{|word| word.include?(letter)}
            if non_matches.size >= half_the_word_list_size
              new_potential_least_worst_case = non_matches.size
            else 
              new_potential_least_worst_case = minimum_of_worst_case(letter,least_worst_case_so_far,matches)
            end

            if new_potential_least_worst_case < least_worst_case_so_far
              least_worst_case_so_far = new_potential_least_worst_case
              @least_worse_case_letter = letter
            end
          end
          @least_worse_case_letter
        end

        def potential_remaining_letters
          @word_list.join.split(//).uniq - already_tried_letters
        end

        def minimum_of_worst_case(letter,best_size_so_far,matches)
          parts = Hash.new{|hash, key| hash[key] = 0}
          
          matches.each do |word|
            masked_word = word.tr('^'+letter,'*')
            parts[masked_word] += 1
            return best_size_so_far if parts[masked_word] > best_size_so_far
          end
          parts.values.last
        end

        def regex_from_word_pattern(word_pattern)
          Regexp.new('^' + word_pattern.gsub(/_/, '\w') + '$', true)
        end

        def words_that_do_not_match(letter)
          @word_list.reject {|word| word.include?(letter)}
        end

        def filter_word_list_by_word_pattern(word_pattern)
          @word_list = @word_list.select do |word|
            regex_from_word_pattern(word_pattern).match word
          end
        end

        def filter_word_list_by_incorrect_letter(guess)
          @word_list = words_that_do_not_match guess
        end
        
        def half_the_word_list_size
          @word_list.size / 2
        end

      end
end