require File.expand_path(File.dirname(__FILE__) + "/../spec_helper")
require 'minimizes_partition/minimizes_partition'

describe MinimizesPartition::MinimizesPartition do
  before(:each) do
    @initial_word_list =  ['cat','cac','cab','hat','one','four','a2','fives']
    @word = '___'
    @player = MinimizesPartition::MinimizesPartition.new
    @player.word_list = @initial_word_list
    @player.new_game(6)

    
  end
  it "should be instantiable with no paramters" do
    lambda { MinimizesPartition::MinimizesPartition.new }.should_not raise_error
  end

  it "should return the initial word list before any guesses" do
    @player.word_list.should == @initial_word_list
  end
  
  it "should respond to guess with a one_character String to try" do
    @player.guess(@word,nil).class.should == String
    @player.guess(@word,nil).size.should == 1
  end
    
  it "a guess should filter initial word list by the size of the word pattern" do
    @player.guess(@word,nil)
    @player.word_list.should == ['cat','cac','cab','hat','one']
  end
  
  it "filters initial word list by a word pattern" do
    @word = '_a_'
    @player.guess(@word,nil)
    @player.word_list.should == ['cat','cac','cab','hat']
  end
  
  it "creates a regex from word pattern" do
    word_pattern = '_a_'
    @player.regex_from_word_pattern(word_pattern).should == /^\wa\w$/i
  end
  
  it "should list letters left in word list that are not already known to be correct" do
    @word = '_a_'
    @player.guess(@word,nil)
    @player.correct_guess('a')
    @player.potential_remaining_letters.sort.should == %w| b c t h |.sort
  end
  
  it "should list letters left in word list that are not already known to be incorrect" do
    @player.guess(@word,nil)
    @player.incorrect_guess('h')
    @player.potential_remaining_letters.sort.should == %w| c a t b o n e |.sort
  end
  
  it "should track correct and incorrect already_tried_letters" do
    @player.already_tried_letters.should == []
    @player.correct_guess('a')
    @player.already_tried_letters.should == ['a']
    @player.incorrect_guess('h')
    @player.already_tried_letters.should == ['a','h']
  end
  
  it "should remove words from word list that match incorrect guesses" do
    @player.guess(@word,nil)
    @player.incorrect_guess('c')
    @player.word_list.should == ['hat','one']
  end

  it "count partition size that don't match a letter" do
    @player.word_list = ['cat','cac','cab','hat','one']
    @player.new_game(nil)
    @player.words_that_do_not_match('o').size.should == 4
    @player.words_that_do_not_match('c').size.should == 2
    @player.words_that_do_not_match('a').size.should == 1
    @player.words_that_do_not_match('t').size.should == 3
    @player.words_that_do_not_match('b').size.should == 4
    @player.words_that_do_not_match('z').size.should == 5
  end
  
  xit "lists letter from word list with least (smallest) worst case (largest partition)" do
    @player.word_list = ['cat','cac','cab','hat','bat']
    @player.new_game(nil)    
    @player.potential_remaining_letters.should == %w| a c t b h |
    @player.letter_with_least_worst_case.should == 'c'
  end
  
  xit "Another lists letter from word list with least (smallest) worst case (largest partition)" do
    @player.word_list = ['cg1','c23','cg4','56g']
    @player.new_game(nil)
    @player.potential_remaining_letters.sort.should == %w| c g 1 2 3 4 5 6 |.sort
    @player.letter_with_least_worst_case.should == 'g'
  end
  
  xit "lists letters left from word list in order of most frequent" do 
    @player.word_list= ['a','cc','bbb','eeee','fffff','dddddd']
    @player.new_game(nil)
    @player.potential_remaining_letters.should == %w| d f e b c a |
  end

  it "should respond to guess with the letter that has smallest large partition" do
    @player.guess(@word,nil).should == 'c'
  end

  it "should not break when playing more than one game" do
    @initial_word_list =  ['cat','cac','cab','hat','one','four','a2','fives','afbc','xyfz']
    @word = '___'
    @player = MinimizesPartition::MinimizesPartition.new
    @player.new_game(nil)
    @player.word_list = @initial_word_list
    @player.new_game(nil)
    @player.guess(@word,nil).should == 'c'
    @word = '____'
    @player.new_game(nil)
    @player.guess(@word,nil)
    @player.word_list.should == ['four','afbc','xyfz']
    @player.new_game(nil)
    @player.guess(@word,nil).should == 'f'
  end
end